@@ -10,11 +10,11 @@ CatalystX::Controller::Auth - A config-driven Catalyst authentication controller
 
 =head1 VERSION
 
-Version 0.18
+Version 0.19
 
 =cut
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 $VERSION = eval $VERSION;
 
@@ -26,12 +26,17 @@ use HTML::FormHandlerX::Form::Login;
 has form_handler                         => ( is => 'ro', isa => 'Str',  default => 'HTML::FormHandlerX::Form::Login' );
 
 has view                                 => ( is => 'ro', isa => 'Str',  default => 'TT' );
-has model                                => ( is => 'ro', isa => 'Str',  default => 'DB::User' );
+
+has realm                                => ( is => 'ro', isa => 'Str',  default => 'default' );
+
+has login_fields                         => ( is => 'ro', isa => 'HashRef', default => sub { { default => [ qw(username password) ] } } );
 
 has login_id_field                       => ( is => 'ro', isa => 'Str',  default => 'username' );
 has login_id_db_field                    => ( is => 'ro', isa => 'Str',  default => 'username' );
+has db_id_field                          => ( is => 'ro', isa => 'Str',  default => 'id'       );
 
 has enable_register                      => ( is => 'ro', isa => 'Bool', default => 1 );
+has enable_sending_register_email        => ( is => 'ro', isa => 'Bool', default => 1 );
 
 has register_template                    => ( is => 'ro', isa => 'Str',  default => 'auth/register.tt'        );
 has login_template                       => ( is => 'ro', isa => 'Str',  default => 'auth/login.tt'           );
@@ -116,6 +121,7 @@ Configure it as you like ...
          login_id_db_field                      email
  	 
          enable_register                        1
+         enable_sending_register_email          1
  	 
          register_template                      auth/register.tt
          login_template                         auth/login.tt
@@ -179,7 +185,7 @@ If you wish to chain off any other mid-point, and/or change the C<PathPart> (by
 
 sub base :Chained('/base') :PathPart('') :CaptureArgs(0)
 {
-	my ( $self, $c ) = @_;
+    my ( $self, $c ) = @_;
 }
 
 =head2 authenticated ( mid-point: / )
@@ -192,9 +198,9 @@ Chain off this action to make sure the user is logged in.
 
 sub authenticated :Chained('base') :PathPart('') :CaptureArgs(0)
 {
-	my ( $self, $c ) = @_;
-	
-	$self->not_authenticated( $c ) if ! $c->user_exists;
+    my ( $self, $c ) = @_;
+    
+    $self->not_authenticated( $c ) if ! $c->user_exists;
 }
 
 =head2 not_authenticated
@@ -209,14 +215,28 @@ An instance method that is also passed the Catalyst context object C<$c>.
 
 sub not_authenticated
 {
-	my ( $self, $c ) = @_;
-
-	$c->response->redirect( $c->uri_for( $self->action_for('login'), { mid => $c->set_error_msg( $self->login_required_message ) } ) );
-	$c->detach;	
+    my ( $self, $c ) = @_;
+    
+    $c->response->redirect( $c->uri_for( $self->action_for('login'), { mid => $c->set_error_msg( $self->login_required_message ) } ) );
+    $c->detach;	
 }
 
 =head2 register ( end-point: /register )
 
+
+
+
+B<NOTE:> There is currently an issue with L<Catalyst::Authentication::Store::DBIx::Class::User> attempting to call C<auto_create()> when the backend resultset is L<DBIx::Class::ResultSet>.
+
+If you are using this backend and see something along the lines of...
+
+ Can not locate object method "auto_create" via package "DBIx::Class::ResultSet" at /usr/lib/perl5/site_perl/5.8.8/Catalyst/Authentication/Store/DBIx/Class/User.pm line 241.
+
+You will need to modify L<Catalyst::Authentication::Store::DBIx::Class::User> to call C<create()> on the L<DBIx::Class::ResultSet> instead of C<auto_create()>.
+
+
+
+
 Register, unless the C<enable_register> option has been turned off (on by default).
 
 If the user is already logged in, it redirects (and detaches) to the URI for C<action_after_login> with
@@ -226,51 +246,69 @@ status message C<already_logged_in_message>.
 
 sub register :Chained('base') :PathPart :Args(0)
 {
-	my ( $self, $c ) = @_;
+    my ( $self, $c ) = @_;
+    
+    if ( ! $self->enable_register )
+    {
+        $c->res->redirect('/');
+        $c->detach;
+    }
+    
+    if ( $c->user_exists )
+    {
+        $c->response->redirect( $c->uri_for_action( $self->action_after_login, { mid => $c->set_status_msg( $self->already_logged_in_message ) } ) );
+        $c->detach;
+    }
+    
+    my $form = $self->form_handler->new( active => [ $self->login_id_field, 'password', 'confirm_password' ] );
+    
+    if ( $c->req->method eq 'POST' )
+    {
+        $form->process( params => $c->request->params );
 
-	if ( ! $self->enable_register )
-	{
-		$c->res->redirect('/');
-		$c->detach;
-	}
-	
-	if ( $c->user_exists )
-	{
-		$c->response->redirect( $c->uri_for_action( $self->action_after_login, { mid => $c->set_status_msg( $self->already_logged_in_message ) } ) );
-		$c->detach;
-	}
+        if ( $form->validated )
+        {
+            my $auth_store = $c->get_auth_realm( $self->realm )->store;
 
-	my $form = $self->form_handler->new( active => [ $self->login_id_field, 'password', 'confirm_password' ] );
-	
-	if ( $c->req->method eq 'POST' )
-	{
-		$form->process( params => $c->request->params );
+            if ( $auth_store->find_user( { $self->login_id_db_field => $form->field( $self->login_id_field )->value }, $c ) )
+            {
+                $c->stash( error_msg => $self->register_exists_failed_message );
+            }
+            else
+            {
+                my $user;
 
-		if ( $form->validated )
-		{
-			if ( $c->model( $self->model )->search( { $self->login_id_db_field => $form->field( $self->login_id_field )->value } )->all )
-			{
-				$c->stash( error_msg => $self->register_exists_failed_message );
-			}
-			else
-			{
-				my $user = $c->model( $self->model )->create( { $self->login_id_db_field => $form->field( $self->login_id_field )->value,
-				                                                password                 => $form->field('password')->value,
-				                                              } );
-	
-				$self->_send_register_email( $c, user => $user );
+                if ( $auth_store->can('auto_create_user') ) 
+                {
+                $c->log->debug( "can auto_create_user");
+                
+                    $user = $auth_store->auto_create_user( { $self->login_id_db_field => $form->field( $self->login_id_field )->value,
+                                                             password                 => $form->field('password')->value,
+                                                           }, $c
+                                                         );
+                                                         $c->log->debug( "done auto-create");
+                }
+                else
+                {
+                    die "Store " . ref($auth_store) . " does not support auto_create_user!";
+                }
 
-				if ( $self->auto_login_after_register )
-				{
-					$c->authenticate( { $self->login_id_db_field => $form->field( $self->login_id_field )->value, password => $form->field('password')->value } );
-				}
-				
-				$self->post_register( $c );
-			}
-		}
-	}
+                if ( $user && $self->enable_sending_register_email )
+                {
+                    $self->send_register_email( $c, user => $user );
+                }
 
-	$c->stash( template => $self->register_template, form => $form );
+                if ( $self->auto_login_after_register )
+                {
+                    $c->authenticate( { $self->login_id_db_field => $form->field( $self->login_id_field )->value, password => $form->field('password')->value }, $self->realm );
+                }
+
+                $self->post_register( $c );
+            }
+        }
+    }
+
+    $c->stash( template => $self->register_template, form => $form );
 }
 
 =head2 send_register_email
@@ -284,40 +322,39 @@ parameteres, specifically the C<user> object.
 
 sub _send_register_email
 {
-	my $self = shift;
-	
-	# legacy method here, just passing through
-	
-	$self->send_register_email( @_ );
-
-	return $self;
+    my $self = shift;
+    
+    # legacy method here, just passing through
+    
+    $self->send_register_email( @_ );
+    
+    return $self;
 }
 
 sub send_register_email
 {
-	my ( $self, $c, %args ) = @_;
-
-	# send registration email to the user
-	
-	$c->stash->{ email_template } = { to           => $args{ user }->email,
-	                                  from         => $self->register_email_from,
-	                                  subject      => $self->register_email_subject,
-	                                  content_type => 'multipart/alternative',
-	                                  templates => [
-	                                                 { template        => $self->register_email_template_plain,
-	                                                   content_type    => 'text/plain',
-	                                                   charset         => 'utf-8',
-	                                                   encoding        => 'quoted-printable',
-	                                                   view            => $self->view, 
-	                                                 }
-	                                               ]
-	};
-        
-        $c->forward( $c->view( $self->register_email_view ) );
-
-	$c->stash( status_msg => "Registration email sent to " . $args{ user }->email );
-
-	return $self;
+    my ( $self, $c, %args ) = @_;
+    
+    # send registration email to the user
+    
+    $c->stash->{ email_template } = { to           => $args{ user }->email,
+                                      from         => $self->register_email_from,
+                                      subject      => $self->register_email_subject,
+                                      content_type => 'multipart/alternative',
+                                      templates => [ { template        => $self->register_email_template_plain,
+                                                       content_type    => 'text/plain',
+                                                       charset         => 'utf-8',
+                                                       encoding        => 'quoted-printable',
+                                                       view            => $self->view, 
+                                                     }
+                                                   ]
+                                    };
+    
+    $c->forward( $c->view( $self->register_email_view ) );
+    
+    $c->stash( status_msg => "Registration email sent to " . $args{ user }->email );
+    
+    return $self;
 }
 
 =head2 post_register
@@ -332,10 +369,10 @@ By default this method redirects to the URI for C<action_after_register> with st
 
 sub post_register
 {
-	my ( $self, $c ) = @_;
-				
-	$c->response->redirect( $c->uri_for_action( $self->action_after_register, { mid => $c->set_status_msg( $self->register_successful_message ) } ) );
-	$c->detach;
+    my ( $self, $c ) = @_;
+    			
+    $c->response->redirect( $c->uri_for_action( $self->action_after_register, { mid => $c->set_status_msg( $self->register_successful_message ) } ) );
+    $c->detach;
 }
 
 =head2 login ( end-point: /login )
@@ -348,43 +385,56 @@ Login, redirect if already logged in.
 
 sub login :Chained('base') :PathPart :Args(0)
 {
-	my ( $self, $c ) = @_;
+    my ( $self, $c ) = @_;
+    
+    if ( $c->user_exists )
+    {
+        $c->response->redirect( $c->uri_for_action( $self->action_after_login, { mid => $c->set_status_msg( $self->already_logged_in_message ) } ) );
+        return;
+    }
 
-	if ( $c->user_exists )
-	{
-		$c->response->redirect( $c->uri_for_action( $self->action_after_login, { mid => $c->set_status_msg( $self->already_logged_in_message ) } ) );
-		return;
-	}
+    my $realm = $c->req->param('realm') || $self->realm;
 
-	my $form = $self->form_handler->new( active => [ $self->login_id_field, 'password' ] );
-	
-	if ( $c->req->method eq 'POST' )
-	{
-		$form->process( params => $c->request->params );
+    my $fields = $self->login_fields->{ $realm };
 
-		if ( $form->validated )
-		{
-			if ( $c->authenticate( { $self->login_id_db_field => $form->field( $self->login_id_field )->value, password => $form->field('password')->value } ) )
-			{
-				if ( $c->req->params->{ remember } )
-	 			{
-					$c->response->cookies->{ remember } = { value => $form->field( $self->login_id_field )->value };
-				}
-				else
-				{
-					$c->response->cookies->{ remember } = { value => '' };
-				}
+    if ( $c->req->param('openid-check') )
+    {
+        ## Returning from openid login, no fields in form for this yet
+        $fields = [];
+    }
 
-				$self->post_login( $c );
-			}
-			else
-			{
-				$c->stash( error_msg => $self->login_failed_message );
-			}
-		}
-	}
+    my $form = $self->form_handler->new( active => $fields );
 
-	$c->stash( template => $self->login_template, form => $form );
+    ## openid returns with GET params!	
+    if( $c->req->param && $c->req->param > 1 ) # at least 2 as we have the "mid" param.. 
+    {
+        $form->process( params => $c->request->params );
+    
+        if ( $form->validated )
+        {
+            my $authinfo = { map { $_ => $form->field( $_ )->value } @$fields };
+            
+            if ( $c->authenticate( $authinfo, $realm ) )
+            {
+                if ( $c->req->params->{ remember } )
+                {
+                    $c->response->cookies->{ remember } = { value => $form->field( $self->login_id_field )->value };
+                }
+                else
+                {
+                    $c->response->cookies->{ remember } = { value => '' };
+                }
+    
+                $self->post_login( $c );
+            }
+            else
+            {
+                $c->stash( error_msg => $self->login_failed_message );
+            }
+        }
+    }
+        
+    $c->stash( template => $self->login_template, form => $form );
 }
 
 =head2 post_login
@@ -399,10 +449,10 @@ By defualt redirects (and detaches) to the URI for C<action_after_login> with a
 
 sub post_login
 {
-	my ( $self, $c ) = @_;
-	
-	$c->response->redirect( $c->uri_for_action( $self->action_after_login, { mid => $c->set_status_msg( $self->login_successful_message ) } ) );
-	$c->detach;
+    my ( $self, $c ) = @_;
+    
+    $c->response->redirect( $c->uri_for_action( $self->action_after_login, { mid => $c->set_status_msg( $self->login_successful_message ) } ) );
+    $c->detach;
 }
 
 =head2 logout ( end-point: /logout )
@@ -415,11 +465,11 @@ Logs out, and redirects back to /login.
 
 sub logout :Chained('base') :PathPart :Args(0)
 {
-	my ( $self, $c ) = @_;
-
-	$c->logout;
-
-	$self->post_logout( $c );
+    my ( $self, $c ) = @_;
+    
+    $c->logout;
+    
+    $self->post_logout( $c );
 }
 
 =head2 post_logout
@@ -434,10 +484,10 @@ By default redirects (and detaches) to the URI for the C<login> action with a st
 
 sub post_logout
 {
-	my ( $self, $c ) = @_;
-	
-	$c->response->redirect( $c->uri_for( $self->action_for( 'login' ), { mid => $c->set_status_msg( $self->logout_successful_message ) } ) );
-	$c->detach;
+    my ( $self, $c ) = @_;
+    
+    $c->response->redirect( $c->uri_for( $self->action_for( 'login' ), { mid => $c->set_status_msg( $self->logout_successful_message ) } ) );
+    $c->detach;
 }
 
 =head2 forgot_password ( end-point: /forgot-password/ )
@@ -450,40 +500,40 @@ Send a forgotten password token to reset it.  This method uses the built-in feat
 
 sub forgot_password :Chained('base') :PathPart('forgot-password') :Args(0)
 {
-	my ( $self, $c ) = @_;
-
-	my $form = $self->form_handler->new( active => [ qw( email ) ] );
-	
-	if ( $c->req->method eq 'POST' )
-	{
-		$form->process( params => $c->request->params );
-
-		if ( $form->validated )
-		{
-		 	my $user = $c->model( $self->model )->find( { $self->login_id_db_field => $c->request->params->{ $self->login_id_field } } );
-
-		 	if ( $user )
-		 	{
-		 		$c->stash( user => $user );
-		 		
-				$form->token_salt( $self->token_salt );
-
-				$form->add_token_field( $self->login_id_field );
-
-				my $token = $form->token;
-
-				$c->stash( token => $token );
-
-				$self->_send_password_reset_email( $c, user => $user );
-			}
-			else
-			{
-				$c->stash( error_msg => $self->forgot_password_id_unknown );
-			}
-		}
-	}
-
-	$c->stash( template => $self->forgot_password_template, form => $form );
+    my ( $self, $c ) = @_;
+    
+    my $form = $self->form_handler->new( active => [ qw( email ) ] );
+    
+    if ( $c->req->method eq 'POST' )
+    {
+        $form->process( params => $c->request->params );
+    
+        if ( $form->validated )
+        {
+            my $user = $c->get_auth_realm($self->realm)->store->find_user( { $self->login_id_db_field => $c->request->params->{ $self->login_id_field } }, $c );
+        
+            if ( $user )
+            {
+                $c->stash( user => $user );
+         		
+                $form->token_salt( $self->token_salt );
+        
+                $form->add_token_field( $self->login_id_field );
+        
+                my $token = $form->token;
+        
+                $c->stash( token => $token );
+        
+                $self->_send_password_reset_email( $c, user => $user );
+            }
+            else
+            {
+                $c->stash( error_msg => $self->forgot_password_id_unknown );
+            }
+        }
+    }
+    
+    $c->stash( template => $self->forgot_password_template, form => $form );
 }
 
 =head2 send_password_reset_email
@@ -497,40 +547,39 @@ parameteres, specifically the C<user> object.
 
 sub _send_password_reset_email
 {
-	my $self = shift;
-	
-	# legacy method here, just passing through
-	
-	$self->send_password_reset_email( @_ );
-
-	return $self;
+    my $self = shift;
+    
+    # legacy method here, just passing through
+    
+    $self->send_password_reset_email( @_ );
+    
+    return $self;
 }
 	
 sub send_password_reset_email
 {
-	my ( $self, $c, %args ) = @_;
-
-	# send reset password username to the user
-	
-	$c->stash->{ email_template } = { to           => $args{ user }->email,
-	                                  from         => $self->forgot_password_email_from,
-	                                  subject      => $self->forgot_password_email_subject,
-	                                  content_type => 'multipart/alternative',
-	                                  templates => [
-	                                                 { template        => $self->forgot_password_email_template_plain,
-	                                                   content_type    => 'text/plain',
-	                                                   charset         => 'utf-8',
-	                                                   encoding        => 'quoted-printable',
-	                                                   view            => $self->view, 
-	                                                 }
-	                                               ]
-	};
-        
-        $c->forward( $c->view( $self->forgot_password_email_view ) );
-
-	$c->stash( status_msg => "Password reset link sent to " . $args{ user }->email );
-
-	return $self;
+    my ( $self, $c, %args ) = @_;
+    
+    # send reset password username to the user
+    
+    $c->stash->{ email_template } = { to           => $args{ user }->email,
+                                      from         => $self->forgot_password_email_from,
+                                      subject      => $self->forgot_password_email_subject,
+                                      content_type => 'multipart/alternative',
+                                      templates => [ { template        => $self->forgot_password_email_template_plain,
+                                                       content_type    => 'text/plain',
+                                                       charset         => 'utf-8',
+                                                       encoding        => 'quoted-printable',
+                                                       view            => $self->view, 
+                                                     }
+                                                   ]
+                                    };
+    
+    $c->forward( $c->view( $self->forgot_password_email_view ) );
+    
+    $c->stash( status_msg => "Password reset link sent to " . $args{ user }->email );
+    
+    return $self;
 }
 
 =head2 reset_password ( end-point: /reset-password/ )
@@ -543,56 +592,56 @@ Reset password using a token sent in an email.
 
 sub reset_password :Chained('base') :PathPart('reset-password') :Args(0)
 {
-	my ( $self, $c ) = @_;
-
-	if ( $c->req->method eq 'GET' && ! $c->request->params->{ token } )
-	{
-		$c->response->redirect( $c->uri_for( $self->action_for('forgot_password'), { mid => $c->set_status_msg("Missing token") } ) );
-		return;
-	}
-	
-	my $form;
-	
-	if ( $c->req->method eq 'GET' )
-	{
-		$form = $self->form_handler->new( active => [ qw( token ) ] );
-
-		$form->token_salt( $self->token_salt );
-
-		$form->add_token_field( $self->login_id_field );
-
-		$form->process( params => { token => $c->request->params->{ token } } );
-		
-		if ( ! $form->validated )
-		{
-			$c->response->redirect( $c->uri_for( $self->action_for('forgot_password'), { mid => $c->set_error_msg("Invalid token") } ) );
-			return;
-		}
-	}
-	
-	if ( $c->req->method eq 'POST' )
-	{
-		$form = $self->form_handler->new( active => [ qw( token password confirm_password ) ] );
-	
-		$form->token_salt( $self->token_salt );
-		
-		$form->add_token_field( $self->login_id_field );
-
-		$form->process( params => $c->request->params );
-
-		if ( $form->validated )
-		{
-			my $user = $c->model( $self->model )->find( { $self->login_id_db_field => $form->field( $self->login_id_field )->value } );
-			
-			$user->password( $form->field('password')->value );
-			
-			$user->update;	
-
-			$self->post_reset_password( $c );
-		}
-	}
-	
-	$c->stash( template => $self->reset_password_template, form => $form );
+    my ( $self, $c ) = @_;
+    
+    if ( $c->req->method eq 'GET' && ! $c->request->params->{ token } )
+    {
+        $c->response->redirect( $c->uri_for( $self->action_for('forgot_password'), { mid => $c->set_status_msg("Missing token") } ) );
+        return;
+    }
+    
+    my $form;
+    
+    if ( $c->req->method eq 'GET' )
+    {
+        $form = $self->form_handler->new( active => [ qw( token ) ] );
+    
+        $form->token_salt( $self->token_salt );
+    
+        $form->add_token_field( $self->login_id_field );
+    
+        $form->process( params => { token => $c->request->params->{ token } } );
+    
+        if ( ! $form->validated )
+        {
+            $c->response->redirect( $c->uri_for( $self->action_for('forgot_password'), { mid => $c->set_error_msg("Invalid token") } ) );
+            return;
+        }
+    }
+    
+    if ( $c->req->method eq 'POST' )
+    {
+        $form = $self->form_handler->new( active => [ qw( token password confirm_password ) ] );
+        
+        $form->token_salt( $self->token_salt );
+        
+        $form->add_token_field( $self->login_id_field );
+        
+        $form->process( params => $c->request->params );
+        
+        if ( $form->validated )
+        {
+            my $user = $c->get_auth_realm($self->realm)->store->find_user( { $self->login_id_db_field => $form->field( $self->login_id_field )->value }, $c );
+            
+            $user->password( $form->field('password')->value );
+            
+            $user->update;	
+            
+            $self->post_reset_password( $c );
+        }
+    }
+    
+    $c->stash( template => $self->reset_password_template, form => $form );
 }
 
 =head2 post_reset_password
@@ -607,11 +656,10 @@ By default redirects (and detaches) to the URI for the C<login> action with a st
 
 sub post_reset_password
 {
-	my ( $self, $c ) = @_;
-
-	$c->response->redirect( $c->uri_for( $self->action_for('login'), { mid => $c->set_status_msg( $self->password_reset_message ) } ) );
-	$c->detach;
-	
+    my ( $self, $c ) = @_;
+    
+    $c->response->redirect( $c->uri_for( $self->action_for('login'), { mid => $c->set_status_msg( $self->password_reset_message ) } ) );
+    $c->detach;
 }
 
 =head2 get ( mid-point: /auth/*/ )
@@ -626,17 +674,19 @@ If no matching user is found, redirects to the URI for the C<login> action.
 
 sub get :Chained('base') :PathPart('auth') :CaptureArgs(1)
 {
-	my ( $self, $c, $id ) = @_;
+    my ( $self, $c, $id ) = @_;
 
-	my $user = $c->model( $self->model )->find( $id );
+    my $auth_store = $c->get_auth_realm( $self->realm )->store;
 
-	if ( ! $user )
-	{
-		$c->response->redirect( $c->uri_for( $self->action_for('login'), { mid => $c->set_status_msg( $self->login_required_message ) } ) );
-		$c->detach;
-	}
+    my $user = $auth_store->find_user( { $self->db_id_field => $id }, $c );
 
-	$c->stash( user => $user );
+    if ( ! $user )
+    {
+        $c->response->redirect( $c->uri_for( $self->action_for('login'), { mid => $c->set_status_msg( $self->login_required_message ) } ) );
+        $c->detach;
+    }
+    
+    $c->stash( user => $user );
 }
 
 =head2 change_password ( end-point: /auth/*/change-password/ )
@@ -649,34 +699,34 @@ Change your password.
 
 sub change_password :Chained('get') :PathPart('change-password') :Args(0)
 {
-	my ( $self, $c ) = @_;
-
-	my $form = $self->form_handler->new( active => [ qw( old_password password confirm_password ) ] );
-	
-	if ( $c->req->method eq 'POST' )
-	{
-		$form->process( params => $c->request->params );
-
-		if ( $form->validated )
-		{
-			my $user = $c->stash->{ user };
-
-			if ( ! $c->authenticate( { $self->login_id_db_field => $user->email, password => $form->field('old_password')->value } ) )
-			{
-				$c->stash( error_msg => 'Old password incorrect' );
-			}
-			else
-			{
-				$user->password( $form->field('password')->value );
-			
-				$user->update;	
-
-				$self->post_change_password( $c );
-			}
-		}
-	}
-
-	$c->stash( template => $self->change_password_template, form => $form );
+    my ( $self, $c ) = @_;
+    
+    my $form = $self->form_handler->new( active => [ qw( old_password password confirm_password ) ] );
+    
+    if ( $c->req->method eq 'POST' )
+    {
+        $form->process( params => $c->request->params );
+        
+        if ( $form->validated )
+        {
+            my $user = $c->stash->{ user };
+            
+            if ( ! $c->authenticate( { $self->login_id_db_field => $user->email, password => $form->field('old_password')->value }, $self->realm ) )
+            {
+                $c->stash( error_msg => 'Old password incorrect' );
+            }
+            else
+            {
+                $user->password( $form->field('password')->value );
+            
+                $user->update;	
+            
+                $self->post_change_password( $c );
+            }
+        }
+    }
+    
+    $c->stash( template => $self->change_password_template, form => $form );
 }
 
 =head2 post_change_password
@@ -691,10 +741,10 @@ By default redirects (and detaches) to the URI for C<action_after_change_passwor
 
 sub post_change_password
 {
-	my ( $self, $c ) = @_;
-				
-	$c->response->redirect( $c->uri_for_action( $self->action_after_change_password, { mid => $c->set_status_msg( $self->password_changed_message ) } ) );
-	$c->detach;
+    my ( $self, $c ) = @_;
+    			
+    $c->response->redirect( $c->uri_for_action( $self->action_after_change_password, { mid => $c->set_status_msg( $self->password_changed_message ) } ) );
+    $c->detach;
 }
 
 =head1 TODO
@@ -745,6 +795,9 @@ L<http://search.cpan.org/dist/CatalystX-Controller-Auth/>
 
 t0m: Tomas Doran E<lt>bobtfish@bobtfish.netE<gt>
 
+castaway: Jess Robinson (OpenID support)
+
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2012 Rob Brown.
